// Copyright (C) 2023 The Qt Company Ltd.
// Copyright (C) 2019 Alexey Edelev <semlanik@gmail.com>
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTPROTOBUFQTGUITYPES_H
#define QTPROTOBUFQTGUITYPES_H

#include <QtProtobufQtGuiTypes/qtprotobufqtguitypesglobal.h>

QT_BEGIN_NAMESPACE

namespace QtProtobuf {

Q_PROTOBUFQTGUITYPES_EXPORT void qRegisterProtobufQtGuiTypes();

}

QT_END_NAMESPACE

#endif // QTPROTOBUFQTGUITYPES_H
