/* This file is autogenerated. DO NOT CHANGE. All changes will be lost */

#ifndef QPROTOBUF_DURATION_H
#define QPROTOBUF_DURATION_H

#include <QtProtobuf/qprotobufmessage.h>
#include <QtProtobuf/qprotobufobject.h>
#include <QtProtobuf/qprotobuflazymessagepointer.h>

#include <QtCore/qbytearray.h>
#include <QtCore/qstring.h>
#include "protobufwellknowntypes_exports.qpb.h"

#include <QtCore/qmetatype.h>
#include <QtCore/qlist.h>
#include <QtCore/qshareddata.h>

#include <memory>


namespace google::protobuf {
class Duration;
using DurationRepeated = QList<Duration>;
namespace Duration_QtProtobufNested {
enum class QtProtobufFieldEnum;
} // namespace Duration_QtProtobufNested


class Duration_QtProtobufData;
class QPB_PROTOBUFWELLKNOWNTYPES_EXPORT Duration : public QProtobufMessage
{
    Q_GADGET
    Q_PROTOBUF_OBJECT
    Q_DECLARE_PROTOBUF_SERIALIZERS(Duration)
    Q_PROPERTY(QtProtobuf::int64 seconds READ seconds WRITE setSeconds SCRIPTABLE false)
    Q_PROPERTY(QtProtobuf::int32 nanos READ nanos WRITE setNanos SCRIPTABLE true)

public:
    using QtProtobufFieldEnum = Duration_QtProtobufNested::QtProtobufFieldEnum;
    Duration();
    ~Duration();
    Duration(const Duration &other);
    Duration &operator =(const Duration &other);
    Duration(Duration &&other) noexcept;
    Duration &operator =(Duration &&other) noexcept;
    bool operator ==(const Duration &other) const;
    bool operator !=(const Duration &other) const;

    QtProtobuf::int64 seconds() const;

    QtProtobuf::int32 nanos() const;
    void setSeconds(const QtProtobuf::int64 &seconds);
    void setNanos(const QtProtobuf::int32 &nanos);
    static void registerTypes();

private:
    QExplicitlySharedDataPointer<Duration_QtProtobufData> dptr;
};
namespace Duration_QtProtobufNested {
Q_NAMESPACE_EXPORT(QPB_PROTOBUFWELLKNOWNTYPES_EXPORT)

enum class QtProtobufFieldEnum {
    SecondsProtoFieldNumber = 1,
    NanosProtoFieldNumber = 2,
};
Q_ENUM_NS(QtProtobufFieldEnum)

} // namespace Duration_QtProtobufNested
} // namespace google::protobuf

Q_DECLARE_METATYPE(google::protobuf::Duration)
#endif // QPROTOBUF_DURATION_H
